@@ -1,6 +1,6 @@
 // This is the implementation of the Chimera class.
 //
-// Copyright (c) 2023 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2024 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt6.
 // 
@@ -161,10 +161,10 @@ Chimera::~Chimera()
 // Register the type and pseudo fully qualified C++ name of a user-defined
 // Python enum.
 void Chimera::registerPyEnum(PyObject *enum_type,
-        const QByteArray &cpp_qualname)
+        const QByteArray &fq_cpp_name)
 {
     Py_INCREF(enum_type);
-    _py_enum_types.insert(enum_type, cpp_qualname);
+    _py_enum_types.insert(enum_type, fq_cpp_name);
 }
 
 
@@ -523,10 +523,10 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
     }
     else if (_py_enum_types.contains((PyObject *)type_obj))
     {
-        metatype = QMetaType(QMetaType::Int);
-
         // Note that we use the fully qualified name.
         _name = _py_enum_types.value((PyObject *)type_obj);
+
+        metatype = QMetaType::fromName(_name);
     }
     else if (type_obj == &PyList_Type)
     {
@@ -829,15 +829,8 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
         break;
 
     case QMetaType::Int:
-        {
-            if (isEnumOrFlag())
-                *reinterpret_cast<QVariant *>(cpp) = QVariant(
-                        get_enum_value(py));
-            else
-                *reinterpret_cast<int *>(cpp) = PyLong_AsLong(py);
-
-            break;
-        }
+        *reinterpret_cast<int *>(cpp) = PyLong_AsLong(py);
+        break;
 
     case QMetaType::UInt:
         *reinterpret_cast<unsigned int *>(cpp) = sipLong_AsUnsignedLong(py);
@@ -1023,6 +1016,10 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
                 }
             }
         }
+        else if (metatype.flags() & QMetaType::IsEnumeration)
+        {
+            *reinterpret_cast<int *>(cpp) = get_enum_value(py);
+        }
         else
         {
             iserr = 1;
@@ -1127,11 +1124,7 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
         break;
 
     case QMetaType::Int:
-        if (isEnumOrFlag())
-        {
-            tmp_storage.tmp_int = get_enum_value(py);
-        }
-        else if (_inexact)
+        if (_inexact)
         {
             // Fit it into the smallest C++ type we can.
 
@@ -1370,6 +1363,10 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
                 variant_data = value_class;
             }
         }
+        else if (metatype.flags() & QMetaType::IsEnumeration)
+        {
+            tmp_storage.tmp_int = get_enum_value(py);
+        }
         else
         {
             // This is a class we don't recognise.
@@ -1565,11 +1562,6 @@ PyObject *Chimera::toPyObject(void *cpp) const
 
     switch (metatype.id())
     {
-    case QMetaType::UnknownType:
-        // In invalid QMetaType probably means a user-defined enum.
-        py = get_enum_key(*reinterpret_cast<int *>(cpp));
-        break;
-
     case QMetaType::Nullptr:
         py = Py_None;
         Py_INCREF(py);
@@ -1584,10 +1576,6 @@ PyObject *Chimera::toPyObject(void *cpp) const
         {
             py = sipConvertFromType(cpp, _type, 0);
         }
-        else if (isEnumOrFlag())
-        {
-            py = get_enum_key(*reinterpret_cast<int *>(cpp));
-        }
         else
         {
             py = PyLong_FromLong(*reinterpret_cast<int *>(cpp));
@@ -1703,6 +1691,10 @@ PyObject *Chimera::toPyObject(void *cpp) const
                     metatype.destroy(copy);
             }
         }
+        else if (metatype.flags() & QMetaType::IsEnumeration)
+        {
+            py = get_enum_key(*reinterpret_cast<int *>(cpp));
+        }
         else if (_name.contains("_QMLTYPE_"))
         {
             // These correspond to objects defined in QML.  We assume that they
@@ -1830,52 +1822,36 @@ bool Chimera::isEnumOrFlag() const
 }
 
 
-// Return the value of an enum, either a wrapped one or a user-defined one.
+// Return the value of a user-defined enum.
 int Chimera::get_enum_value(PyObject *py) const
 {
-    int value = 0;
+    PyObject *value_obj = PyObject_GetAttrString(py, "value");
 
-    if (_type)
-    {
-        value = sipConvertToEnum(py, _type);
-    }
-    else
-    {
-        PyObject *value_obj = PyObject_GetAttrString(py, "value");
+    if (!value_obj)
+        return 0;
 
-        if (value_obj)
-        {
-            value = PyLong_AsLong(value_obj);
-            Py_DECREF(value_obj);
-        }
-    }
+    int value = PyLong_AsLong(value_obj);
+
+    Py_DECREF(value_obj);
 
     return value;
 }
 
 
-// Return the key of an enum, either a wrapped one or a user-defined one.
+// Return the key of a user-defined enum.
 PyObject *Chimera::get_enum_key(int cpp) const
 {
     PyObject *py = NULL;
+    QHashIterator<PyObject *, QByteArray> it(_py_enum_types);
 
-    if (_type)
+    while (it.hasNext())
     {
-        py = sipConvertFromEnum(cpp, _type);
-    }
-    else
-    {
-        QHashIterator<PyObject *, QByteArray> it(_py_enum_types);
+        it.next();
 
-        while (it.hasNext())
+        if (it.value() == _name)
         {
-            it.next();
-
-            if (it.value() == _name)
-            {
-                py = PyObject_CallFunction(it.key(), "(i)", cpp);
-                break;
-            }
+            py = PyObject_CallFunction(it.key(), "(i)", cpp);
+            break;
         }
     }
 
